@@ -11,12 +11,13 @@
 
 #include <QObject>
 
-#include "types.h"
 #include "bluezqt_export.h"
+#include "types.h"
+
+#include <memory>
 
 namespace BluezQt
 {
-
 class Adapter;
 class PendingCall;
 
@@ -45,6 +46,8 @@ class BLUEZQT_EXPORT Device : public QObject
     Q_PROPERTY(bool blocked READ isBlocked WRITE setBlocked NOTIFY blockedChanged)
     Q_PROPERTY(bool legacyPairing READ hasLegacyPairing NOTIFY legacyPairingChanged)
     Q_PROPERTY(qint16 rssi READ rssi NOTIFY rssiChanged)
+    Q_PROPERTY(ManData manufacturerData READ manufacturerData NOTIFY manufacturerDataChanged)
+    Q_PROPERTY(bool servicesResolved READ isServicesResolved NOTIFY servicesResolvedChanged)
     Q_PROPERTY(bool connected READ isConnected NOTIFY connectedChanged)
     Q_PROPERTY(QStringList uuids READ uuids NOTIFY uuidsChanged)
     Q_PROPERTY(QString modalias READ modalias NOTIFY modaliasChanged)
@@ -53,6 +56,7 @@ class BLUEZQT_EXPORT Device : public QObject
     Q_PROPERTY(MediaPlayerPtr mediaPlayer READ mediaPlayer NOTIFY mediaPlayerChanged)
     Q_PROPERTY(MediaTransportPtr mediaTransport READ mediaTransport NOTIFY mediaTransportChanged)
     Q_PROPERTY(AdapterPtr adapter READ adapter)
+    Q_PROPERTY(QList<GattServiceRemotePtr> gattServices READ gattServices NOTIFY gattServicesChanged)
 
 public:
     /**
@@ -81,7 +85,7 @@ public:
         Joypad,
         /** The device is a graphics tablet (input device). */
         Tablet,
-        /** The deivce is an uncategorized peripheral device. */
+        /** The device is an uncategorized peripheral device. */
         Peripheral,
         /** The device is a camera. */
         Camera,
@@ -96,14 +100,14 @@ public:
         /** The device is a health device. */
         Health,
         /** The device is not of any of the known types. */
-        Uncategorized
+        Uncategorized,
     };
     Q_ENUM(Type)
 
     /**
      * Destroys a Device object.
      */
-    ~Device();
+    ~Device() override;
 
     /**
      * Returns a shared pointer from this.
@@ -261,6 +265,23 @@ public:
      */
     qint16 rssi() const;
 
+    /**
+     * Returns manufacturer specific advertisement data.
+     *
+     * @note Keys are 16 bits Manufacturer ID followed by
+     * its byte array value.
+     *
+     * @return manufacturerData of device.
+     */
+    ManData manufacturerData() const;
+
+    /**
+     * Returns whether or not service discovery has been resolved.
+     *
+     * @return true if servicesResolved
+     */
+    bool isServicesResolved() const;
+
     /**
      * Returns whether the device is connected.
      *
@@ -284,6 +305,14 @@ public:
      */
     QString modalias() const;
 
+    /**
+     * Returns the service advertisement data.
+     *
+     * @returns A hash with keys being the UUIDs in and values being the raw service data value.
+     * @since 5.72
+     */
+    QHash<QString, QByteArray> serviceData() const;
+
     /**
      * Returns the battery interface for the device.
      *
@@ -328,6 +357,12 @@ public:
      */
     AdapterPtr adapter() const;
 
+    /**
+     * Returns list of services known by the device.
+     *
+     * @return list of services
+     */
+    QList<GattServiceRemotePtr> gattServices() const;
     /**
      * Returns a string for device type.
      *
@@ -424,6 +459,26 @@ Q_SIGNALS:
      */
     void deviceChanged(DevicePtr device);
 
+    /**
+     * Indicates that a new service was added (eg. found by connection).
+     */
+    void gattServiceAdded(GattServiceRemotePtr service);
+
+    /**
+     * Indicates that device GATT services list has changed
+     */
+    void gattServicesChanged(QList<GattServiceRemotePtr> services);
+
+    /**
+     * Indicates that a service was removed.
+     */
+    void gattServiceRemoved(GattServiceRemotePtr service);
+
+    /**
+     * Indicates that at least one of the device's services have changed.
+     */
+    void gattServiceChanged(GattServiceRemotePtr service);
+
     /**
      * Indicates that device's name have changed.
      */
@@ -489,6 +544,16 @@ Q_SIGNALS:
      */
     void rssiChanged(qint16 rssi);
 
+    /**
+     * Indicates that device's manufacturer data have changed.
+     */
+    void manufacturerDataChanged(ManData man);
+
+    /**
+     * Indicates that device's servicesResolved state have changed.
+     */
+    void servicesResolvedChanged(bool servicesResolved);
+
     /**
      * Indicates that device's connected state have changed.
      */
@@ -504,6 +569,12 @@ Q_SIGNALS:
      */
     void modaliasChanged(const QString &modalias);
 
+    /**
+     * Indicates that the device's service data has changed.
+     * @since 5.72
+     */
+    void serviceDataChanged(const QHash<QString, QByteArray> &serviceData);
+
     /**
      * Indicates that device's battery has changed.
      */
@@ -525,9 +596,9 @@ Q_SIGNALS:
     void mediaTransportChanged(MediaTransportPtr mediaTransport);
 
 private:
-    explicit Device(const QString &path, const QVariantMap &properties, AdapterPtr adapter);
+    BLUEZQT_NO_EXPORT explicit Device(const QString &path, const QVariantMap &properties, AdapterPtr adapter);
 
-    class DevicePrivate *const d;
+    std::unique_ptr<class DevicePrivate> const d;
 
     friend class DevicePrivate;
     friend class ManagerPrivate;
@@ -536,4 +607,6 @@ private:
 
 } // namespace BluezQt
 
+Q_DECLARE_METATYPE(BluezQt::ManData)
+
 #endif // BLUEZQT_DEVICE_H
